@@ -4,7 +4,7 @@
 # Univention Management Console
 #  module: password reset service
 #
-# Copyright 2015-2021 Univention GmbH
+# Copyright 2015-2022 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -52,6 +52,7 @@ from univention.lib.umc import Client, HTTPError, ConnectionError, Unauthorized
 import univention.admin.objects
 import univention.admin.syntax
 import univention.admin.uexceptions as udm_errors
+from univention.admin.uldap import getMachineConnection
 from univention.management.console.modules import Base
 from univention.management.console.log import MODULE
 from univention.management.console.config import ucr
@@ -70,18 +71,18 @@ MEMCACHED_MAX_KEY = 250
 
 SELFSERVICE_MASTER = ucr.get("self-service/backend-server", ucr.get("ldap/master"))
 IS_SELFSERVICE_MASTER = '%s.%s' % (ucr.get('hostname'), ucr.get('domainname')) == SELFSERVICE_MASTER
+UDM_REST_SERVER = ucr.get('self-service/udm-rest-server', '%s.%s' % (ucr.get('hostname'), ucr.get('domainname')))
 DISALLOW_AUTHENTICATION = not ucr.is_true('umc/self-service/allow-authenticated-use')
 
 DEREGISTRATION_TIMESTAMP_FORMATTING = '%Y%m%d%H%M%SZ'
 
 if IS_SELFSERVICE_MASTER:
 	try:
-		from univention.management.console.modules.udm.syntax import widget
 		from univention.management.console.modules.udm.udm_ldap import UDM_Error, UDM_Module
 		from univention.udm import UDM, NoObject
+		from univention.admin.rest.client import UDM as UDMRest
 	except ImportError as exc:
 		MODULE.error('Could not load udm module: %s' % (exc,))
-		widget = None
 
 
 def forward_to_master(func):
@@ -272,6 +273,62 @@ class Instance(Base):
 				univention.admin.modules.init(lo, po, self._usersmod)
 		return self._usersmod
 
+	@forward_to_master
+	@sanitize(
+		username=StringSanitizer(required=True, minimum=1),
+		password=StringSanitizer(required=True, minimum=1))
+	@simple_response
+	def get_service_specific_passwords(self, username, password):
+		"""
+		Get users (possible) service specific passwords.
+
+		:return: list of dicts with users ssp
+		"""
+		if ucr.is_false('umc/self-service/service-specific-passwords/backend/enabled'):
+			msg = _('Service specific passwords were disabled via the Univention Configuration Registry.')
+			MODULE.error('get_service_specific_passwords(): {}'.format(msg))
+			raise UMC_Error(msg)
+
+		dn, username = self.auth(username, password)
+		ret = []
+		# we only have radius as a service specific password: setting
+		# the backend to true means that the admin wants to allow
+		# radius password. at some point we would need to know which
+		# services should actually be managed
+		ldap_connection, ldap_position = getMachineConnection()
+		radius_passwords = ldap_connection.get(dn, attr=['univentionRadiusPassword']).get('univentionRadiusPassword', [])
+		ret.append({'type': 'radius', 'set': len(radius_passwords)})
+		return ret
+
+	@forward_to_master
+	@sanitize(
+		username=StringSanitizer(required=True, minimum=1),
+		password=StringSanitizer(required=True, minimum=1),
+		password_type=StringSanitizer(required=True, minimum=1))
+	@simple_response
+	def set_service_specific_passwords(self, username, password, password_type):
+		'''
+		Set a new service specific password.
+
+		:return: The password in cleartext
+		'''
+		if ucr.is_false('umc/self-service/service-specific-passwords/backend/enabled'):
+			msg = _('Service specific passwords were disabled via the Univention Configuration Registry.')
+			MODULE.error('get_service_specific_passwords(): {}'.format(msg))
+			raise UMC_Error(msg)
+
+		dn, username = self.auth(username, password)
+		MODULE.error('set_service_specific_passwords(): Setting {} password for {}'.format(password_type, username))
+		if password_type == 'radius':
+			udm = UDMRest.http('https://%s/univention/udm/' % UDM_REST_SERVER, 'cn=admin', open('/etc/ldap.secret').read())
+			user_obj = udm.get('users/user').get(dn)
+			service_specific_password = user_obj.generate_service_specific_password('radius')
+		else:
+			msg = _('Service specific passwords were disabled for "%s".') % password_type
+			MODULE.error('get_service_specific_passwords(): {}'.format(msg))
+			raise UMC_Error(msg)
+		return {'password': service_specific_password}
+
 	@forward_to_master
 	@sanitize(
 		username=StringSanitizer(required=True, minimum=1),
@@ -376,7 +433,7 @@ class Instance(Base):
 				'readonly': not bool(prop.editable),
 				'multivalue': bool(prop.multivalue),
 			}
-			widget_description.update(widget(prop.syntax, widget_description))
+			widget_description.update(prop.syntax.get_widget_options(prop))
 			if 'udm' in widget_description['type']:
 				continue
 			if 'dynamicValues' in widget_description:
@@ -620,8 +677,17 @@ class Instance(Base):
 			self._usersmod = None
 			#  univention.admin.modules.init(lo, po, usersmod, None, True)
 		try:
-			self.send_message(new_user['username'], 'verify_email', new_user['PasswordRecoveryEmail'], raise_on_success=False)
+			# in all SS cases we need more than the previously default fields
+			user_info = self._extract_user_properties(new_user)
+			self.send_message(
+				new_user['username'],
+				'verify_email',
+				new_user['PasswordRecoveryEmail'],
+				user_info,
+				raise_on_success=False
+			)
 		except Exception:
+			MODULE.error('could not send message: %s' % (traceback.format_exc(),))
 			verify_token_successfully_send = False
 		else:
 			verify_token_successfully_send = True
@@ -634,6 +700,21 @@ class Instance(Base):
 			}
 		}
 
+	def _extract_user_properties(self, user_obj):
+		message_fields = [
+			'username',
+			'title',
+			'initials',
+			'displayName',
+			'organisation',
+			'employeeNumber',
+			'firstname',
+			'lastname',
+			'mailPrimaryAddress'
+		]
+		info_out = {field: user_obj.info.get(field, '') for field in message_fields}
+		return info_out
+
 	@forward_to_master
 	@prevent_denial_of_service
 	@sanitize(
@@ -662,7 +743,8 @@ class Instance(Base):
 		else:
 			if not email:
 				return invalid_information
-		self.send_message(username, 'verify_email', email, raise_on_success=False)
+		user_info = self._extract_user_properties(user._orig_udm_object)
+		self.send_message(username, 'verify_email', email, user_info, raise_on_success=False)
 		return {
 			'success': True,
 			'data': {
@@ -719,7 +801,8 @@ class Instance(Base):
 
 		if len(user[plugin.udm_property]) > 0:
 			# found contact info
-			self.send_message(username, method, user[plugin.udm_property])
+			user_info = self._extract_user_properties(user)
+			self.send_message(username, method, user[plugin.udm_property], user_info, raise_on_success=True)
 
 		# no contact info
 		raise MissingContactInformation()
@@ -923,7 +1006,7 @@ class Instance(Base):
 		rand = random.SystemRandom()
 		return ''.join(rand.choice(chars) for _ in range(length))
 
-	def send_message(self, username, method, address, raise_on_success=True):
+	def send_message(self, username, method, address, user_properties, raise_on_success=True):
 		plugin = self._get_send_plugin(method)
 		try:
 			token_from_db = self.db.get_one(username=username)
@@ -943,7 +1026,7 @@ class Instance(Base):
 			MODULE.info("send_token(): Adding new token for user '{}'...".format(username))
 			self.db.insert_token(username, method, token)
 		try:
-			self._call_send_msg_plugin(username, method, address, token)
+			self._call_send_msg_plugin(username, method, address, token, user_properties)
 		except Exception:
 			MODULE.error("send_token(): Error sending token with via '{method}' to '{username}'.".format(
 				method=method, username=username))
@@ -963,14 +1046,16 @@ class Instance(Base):
 			raise UMC_Error("Unknown send message method", status=500)
 		return plugin
 
-	def _call_send_msg_plugin(self, username, method, address, token):
+	def _call_send_msg_plugin(self, username, method, address, token, user_properties):
 		MODULE.info("send_message(): username: {} method: {} address: {}".format(username, method, address))
 		plugin = self._get_send_plugin(method)
 
 		plugin.set_data({
 			"username": username,
 			"address": address,
-			"token": token})
+			"token": token,
+			"user_properties": user_properties,
+		})
 		MODULE.info("send_message(): Running plugin of class {}...".format(plugin.__class__.__name__))
 		try:
 			plugin.send()
@@ -1020,7 +1105,8 @@ class Instance(Base):
 				if old_email is not None and old_email.lower() != email.lower():
 					self._notify_about_email_change(user['username'], old_email, email)
 				if email is not None and email.lower() != old_email.lower():
-					self.send_message(user['username'], 'verify_email', email, raise_on_success=False)
+					user_info = self._extract_user_properties(user)
+					self.send_message(user['username'], 'verify_email', email, user_info, raise_on_success=False)
 					verification_email_send = True
 			return {
 				'verificationEmailSend': verification_email_send,
