@@ -3,7 +3,7 @@
 #
 # Send a token to a user by email.
 #
-# Copyright 2015-2021 Univention GmbH
+# Copyright 2015-2022 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -104,10 +104,16 @@ class SendEmail(UniventionSelfServiceTokenEmitter):
 
 		fqdn = ".".join([self.ucr["hostname"], self.ucr["domainname"]])
 		frontend_server = self.ucr.get("umc/self-service/passwordreset/email/webserver_address", fqdn)
-		link = "https://{fqdn}/univention/self-service/#page=newpassword".format(fqdn=frontend_server)
-		tokenlink = "https://{fqdn}/univention/self-service/#page=newpassword&token={token}&username={username}".format(fqdn=frontend_server, username=quote(self.data["username"]), token=quote(self.data["token"]))
+		links = {
+			'link': "https://{fqdn}/univention/selfservice/#/selfservice/newpassword/".format(fqdn=frontend_server),
+			'tokenlink': "https://{fqdn}/univention/selfservice/#/selfservice/newpassword/?token={token}&username={username}".format(fqdn=frontend_server, username=quote(self.data["username"]), token=quote(self.data["token"]))
+		}
 
-		txt = txt.format(username=self.data["username"], token=self.data["token"], link=link, tokenlink=tokenlink)
+		formatter_dict = self.data['user_properties']
+		formatter_dict.update(links)
+		formatter_dict['token'] = self.data['token']
+
+		txt = txt.format(**formatter_dict)
 
 		msg = MIMENonMultipart('text', 'plain', charset='utf-8')
 		cs = email.charset.Charset("utf-8")
